/**
 * $Id: scanner_rfc.cpp,v 1.7 2003/02/25 19:49:29 plasmahh Exp $
 * $Revision: 1.7 $
 * $Author: plasmahh $
 * $Date: 2003/02/25 19:49:29 $
 */

#include "scanner_rfc.h"

scanner_rfc::~scanner_rfc()/*{{{*/
{
    conffp.clear();
    rfcfp.clear();
    returnfp.clear();
    if ( scanned_conf )   delete scanned_conf;
    if ( scanned_rfc )    delete scanned_rfc;
    if ( scanned_return ) delete scanned_return;
}/*}}}*/

bool scanner_rfc::initialize (void)/*{{{*/
{
    if ( debuglevel > 1 )
    {
        cout << "Reading fingerprints from file..." << endl;
    }

    bool ret = true;
    ret &= load_return_fps ();
    ret &= load_rfc_fps ();
    ret &= load_conf_fps ();

    if ( debuglevel > 1 )
    {
        cout << "Done reading fingerprints, error : " << !ret << endl;
    }

    return ret;
}/*}}}*/

void scanner_rfc::finalize (void)/*{{{*/
{
    try {
    if ( debuglevel > 1 )
    {
	cout << "Finalizing fingerprints ..." ;
    }
    // iterate through the fingerprints and set error values
    dlliterator < fp_conf > itc (conffp);
    dlliterator < fp_rfc > itf (rfcfp);
    dlliterator < fp_return > itr (returnfp);

    for (itc.first (); itc.valid (); itc.next ())
    {
	itc.item ().gen_error (*scanned_conf);
    }

    for (itf.first (); itf.valid (); itf.next ())
    {
	itf.item ().gen_error (*scanned_rfc);
    }

    for (itr.first (); itr.valid (); itr.next ())
    {
	itr.item ().gen_error (*scanned_return);
    }
	
    if (debuglevel > 1)
    {
	cout << " done." << endl;
    }
    }
    catch ( Exception & e )
    {
    }
    if ( debuglevel >= 1)
    {
	cout << * scanned_return << endl;
	cout << * scanned_rfc << endl;
	cout << * scanned_conf << endl;
    }
}/*}}}*/

dllist < dllist < fp > >scanner_rfc::get_fps (void)/*{{{*/
{
    /*
     * We return all the fingerprints here in a dllist.
     * 1. Item : dllist of all the config fingerprints we have
     * 2. Item : dllist of all the rfc fingerprints we have.
     * 3. Item : dllist of all the generic return fingerprints.
     *
     * Since we return all of them as FP, not as FP_CONF or whatever, we will only
     * get description and error values !!! Not the real fingerprint, which means
     * we can output it only in this class !!!  this needs us to get the debuglevel
     * in the future
     */
    dlliterator < fp_conf > itc (conffp);
    dlliterator < fp_rfc > itf (rfcfp);
    dlliterator < fp_return > itr (returnfp);

    dllist < fp > fpc, fpf, fpr;

    // iterate through the config fingerprints
    for (itc.first (); itc.valid (); itc.next ())
    {
	fpc.add (itc.item ());
    }

    for (itf.first (); itf.valid (); itf.next ())
    {
	fpf.add (itf.item ());
    }

    for (itr.first (); itr.valid (); itr.next ())
    {
	fpr.add (itr.item ());
    }

    // all have been added, generate the "big" dllist
    dllist < dllist < fp > >allfps;
    allfps.add (fpc);
    allfps.add (fpf);
    allfps.add (fpr);
    return allfps;
    // return a list of lists of fingerprints
}/*}}}*/

bool scanner_rfc::scan_add_to_fp( const mstring&  answer )
{
    scanned_return->add (answer);
    scanned_conf->add (answer);
    scanned_rfc->add (answer);

    return true;
}

bool scanner_rfc::scan_finished( void )
{
    return ( ! actual_command->command );
}
/*
bool scanner_rfc::scan_special( long number )
{
    if (number == 2)
    {
	do_login();

    }
    return true;
}
*/

bool scanner_rfc::scan_init( void ) /*{{{*/
{
    mstring banner;
    if ( debuglevel >= every )
    {
	cout << "scanner_pop3::scan_init()" << endl;
    }
    actual_command = cmd_list;
    recon_count = 0;
    if ( debuglevel >= every )
    {
	cout << "sock.Connect(" << target_host << ", " << target_port << ", " << conn_timeout << ")" << endl;
    }
    if ( ! sock.Connect(target_host, target_port, conn_timeout) )
    {
	cout << " : " << sock.GetLastErrorMsg() << endl;
	return false;
    }
    banner = get_rfc_answer(); // banner
    if ( debuglevel >= every )
    {
	cout << "Creating local Data" << endl;
    }
    if ( scanned_return )
    {
	delete scanned_return;
    }
    if ( scanned_conf )
    {
        delete scanned_conf;
    }
    if ( scanned_rfc )
    {
        delete scanned_rfc;
    }
    scanned_return = new fp_return(banner);
    scanned_conf = new fp_conf(banner);
    scanned_rfc = new fp_rfc(banner);
    return true;
}/*}}}*/

dllist < mstring > scanner_rfc::get_descs (void)/*{{{*/
{
    // return a list of descriptions of the fingerprint types...
    // hardcode this here a bit...
    dllist < mstring > desc;
    desc.add ("Configuration Fingerprinting");
    desc.add ("RFC Fingerprinting");
    desc.add ("Overall Fingerprinting");
    return desc;
}/*}}}*/

bool scanner_rfc::add_fp( fp& n_fp, fp_type f_type)/*{{{*/
{
switch (f_type)
	{
	    case FP_Conf :
		conffp.add ( (fp_conf&)n_fp);
		break;
	    case FP_Return :
		returnfp.add( (fp_return&)n_fp);
		break;
	    case FP_Rfc :
		rfcfp.add((fp_rfc&)n_fp);
		break;
	    case FP_Pop3 :
	    case FP_Ftp  :
	    case FP_Imap :
	    case FP_Base :
		throw x_implemented("Invalid Fingerprint type (implementation failure ?)",__FLF__);
		return false;
		break;
	}
	return true;
}/*}}}*/

#if 0
bool scanner_rfc::do_scan (const mstring& username, const mstring& password)/*{{{*/
{
    // do the actual scan... humm, this will be a lot I guess
    const char **cmd;
    mstring answer;
    mstring temp;
    mstring banner;
    uint32 numbers, count;
    uint32 recon = 0; 
    numbers = cmd_number;
    cmd = cmd_list;
    // connect to target
    if (debuglevel > 2)
    {
	cout << "scanner_smtp calls Connect ( " << target_host << ")" << endl;
    }

    if ( !  sock.Connect (target_host, target_port, conn_timeout) )
    {
//	cout << "Could not connect to  host, aborting scan ! " << endl;
//	cout << "Reason : " ;
	cout << " : " << sock.GetLastErrorMsg() << endl;
	return false;
    }

    banner = get_rfc_answer();

    do_login ( username, password );
// creaeting fingerprints for "local" use...
    if (scanned_return)
    {
        delete scanned_return;
    }
    if (scanned_conf)
    {
        delete scanned_conf;
    }
    if (scanned_rfc)
    {
        delete scanned_rfc;
    }

    scanned_return = new fp_return (banner);
    scanned_conf   = new fp_conf   (banner);
    scanned_rfc    = new fp_rfc    (banner);
    count  = 0;
    cout << endl;

    for ( ; *cmd != NULL; cmd++)
    {

	if ( debuglevel > 1 )
	{
	    cout << "\nSEND " << *cmd;
	} 
	sock.Send (*cmd);
	answer = get_rfc_answer(); // delete the answer...
/// ######################## disconnection detected, need re-login !!!
	if ( ! sock.Connected() )
	{
	    cout << endl << "Disconnection detected, trying to reconnect" << endl;
	// disconnection detected... try to reconnet
	    recon++;
	    if ( recon > 3 )
	    {
	        cout << "too much disconnects detected, aborting this scan" << endl;
		return false;
	    }
	    if ( ! sock.Reconnect(conn_timeout) )
	    {
		cout << "failed to reconnect" << endl;
		return false;
	    }
	    banner = get_rfc_answer();
	    do_login ( username, password );
	    //temp = get_rfc_answer();
	}
/// ############################# re-login done ######################	
	scanned_return->add (answer);
	scanned_conf->add (answer);
	scanned_rfc->add (answer);
	if (debuglevel > 1 )
	{
            cout << "RECV " << answer << endl;
	}
	count ++;
	if ( debuglevel > 2 )
	{
	    cout << count << ". Result Checksum is : " << answer.checksum_sum() << endl;
	}
	cout << "\r" << (double) count / (double) numbers  * 100.0 << " %" << flush;
    }
    cout << "\r100 % done scan" << endl;
    if ( debuglevel > 0)
    {
	cout << "The scanned fingerprints are : " << endl;
	cout << *scanned_return;
	cout << *scanned_rfc;
	cout << *scanned_conf;
    }
    return true;
}/*}}}*/
#endif
/**
 *$Log: scanner_rfc.cpp,v $
 *Revision 1.7  2003/02/25 19:49:29  plasmahh
 *Ok, lets release ;)
 *
 *Revision 1.6  2003/02/25 16:36:35  plasmahh
 *A lot of changes ;)
 *
 *Revision 1.5  2002/11/25 23:33:35  plasmahh
 *changed a bit stuff and new fingerprints and and and ;)
 *
 *Revision 1.4  2002/11/05 20:56:27  plasmahh
 *new stuff, now ftp should work quite good
 *
 *Revision 1.2  2002/11/05 00:31:10  plasmahh
 *fixed rfc fingerprinting
 *
 *Revision 1.1  2002/11/04 23:45:52  plasmahh
 *first addition, implementation and changes for ftp fingerprinting
 *rfc fp fix still mising !
 *
 *Revision 1.26  2002/11/03 22:25:58  plasmahh
 *new stuff for better rfc fingerprinting and preperation for ftp support... other protocolls might follow
 *
 *Revision 1.25  2002/10/21 08:38:29  plasmahh
 *begin cleanup and structure changes for smtpmap 0.8-stable
 *
 *Revision 1.24  2002/10/16 23:06:17  plasmahh
 *new function for reading fingerprints from file
 *fixes some stuff
 *
 *Revision 1.23  2002/10/01 20:59:02  plasmahh
 *some new structures, to maintain multiple protocols bette
 *
 *Revision 1.22  2002/08/12 13:13:05  plasmahh
 *removed some memory leaks
 *
 *Revision 1.21  2002/08/08 21:52:59  plasmahh
 *better debug
 *
 *Revision 1.20  2002/08/08 21:47:28  plasmahh
 *more debug
 *
 *Revision 1.19  2002/07/24 18:06:47  plasmahh
 *some slight fixes
 *fingerprint changes and cleanups
 *default value changes
 *
 *Revision 1.18  2002/06/20 20:37:01  plasmahh
 *new fingerprints
 *fixed correct fingerprinting on multiple hosts
 *
 *Revision 1.17  2002/06/18 13:42:39  plasmahh
 *fixed timeout behaviour
 *new fingerprints added
 *improved documentation
 *
 *Revision 1.16  2002/06/17 12:23:48  plasmahh
 *fixed timeout bugs
 *
 *Revision 1.14  2002/06/12 23:05:07  plasmahh
 *New fingerprints
 *
 *Revision 1.13  2002/06/11 21:39:31  plasmahh
 *improved reconnection behaviour
 *
 *Revision 1.12  2002/06/11 21:01:53  plasmahh
 *forgot debug messages
 *
 *Revision 1.11  2002/06/11 20:57:38  plasmahh
 *hopefully fixed "hanging" bug
 *
 *Revision 1.10  2002/06/11 20:04:15  plasmahh
 *fixed bad connection behaviour bug
 *
 *Revision 1.9  2002/06/11 18:54:35  plasmahh
 *fixed timeout behaviour, need only to fix reconnects...
 *new fingerprint
 *
 *Revision 1.8  2002/06/11 18:23:53  plasmahh
 *reverified fingerprints
 *
 *Revision 1.7  2002/06/09 21:22:39  plasmahh
 *lots of changes
 *some slight fingerprint changes, also in constructor
 *scanner changes and fixes in constructor
 *read fixes
 *test programs fixes, no more segfaults
 *new TODO
 *
 *Revision 1.6  2002/05/29 12:10:35  plasmahh
 *hm, ja... dont know ;)
 *
 *Revision 1.5  2002/05/27 06:26:17  plasmahh
 *Implemented, tested and fixed fingerprinting. Test Programs generate good fingerprints, only need to put all in a program and generateerror values
 *
 *Revision 1.4  2002/05/23 22:37:55  plasmahh
 *fixed small things in scanner
 *split fps into .cpp files, because of linker errors in operators
 *
 *Revision 1.3  2002/05/23 19:50:04  plasmahh
 *refined include structure.
 *implemented and fixed some parts, still not working
 *BUG !! FIX THIS !!! : multiple definition of some operators, we will never compile !!
 *
 *Revision 1.2  2002/05/16 23:32:12  plasmahh
 *Added fingerprint reading, first version needs to be heavily tested !
 *
 *Revision 1.1  2002/05/14 16:31:59  plasmahh
 *implementing new scanner structure, that will be   e xtendible
 *
 */
